<?php

declare(strict_types=1);

namespace mirzaev\calculator\models;

use mirzaev\calculator\models\settings_model as settings;

use exception;

/**
 * Модель калькуляторов
 *
 * @package mirzaev\calculator\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class calculators_model extends core
{
    /**
     * Рассчет калькулятора лазерной резки
     *
     * @param bool|int|string|null $company Юридическое лицо? Или физическое лицо?
     * @param string|null $complexity Сложность ('easy', 'medium', 'hard')
     * @param int|string|null $width Высота (мм)
     * @param int|string|null $height Ширина (мм)
     * @param int|string|null $lenght Длина (мм)
     * @param int|string|null $amount Количество
     * @param string|null $metal Тип металла
     * @param int|string|null $holes Количество отверстий
     * @param int|string|null $diameter Диаметр отверстий (мм)
     * @param array &$errors Журнал ошибок
     *
     * @return array|bool Аккаунт, если удалось аутентифицироваться
     *
     * @todo
     * 1. Значения по умолчанию брать из настроек в базе данных
     */
    public static function laser(
        bool|int|string|null $company = null,
        string|null $complexity = null,
        int|string|null $width = null,
        int|string|null $height = null,
        int|string|null $length = null,
        int|string|null $amount = null,
        string|null $metal = null,
        int|string|null $holes = null,
        int|string|null $diameter = null,
        array &$errors = []
    ): array {
        try {
            // Инициализация переменных для буфера вывода
            $machine = $manager = $engineer = $operator = 0;

            // Инициализация значений по умолчанию (см. @todo)
            $company = (bool) ($company ?? settings::read('default_buyer', $errors) === 'entity' ?? false);
            $complexity = (string) ($complexity ?? settings::read('default_complexity', $errors) ?? 'medium');
            $width = (int) ($width ?? settings::read('default_width', $errors) ?? 500);
            $height = (int) ($height ?? settings::read('default_height', $errors) ?? 500);
            $length = (int) ($length ?? settings::read('default_length', $errors) ?? 1);
            $amount = (int) ($amount ?? settings::read('default_amount', $errors) ?? 1);
            $metal = (string) ($metal ?? settings::read('default_metal', $errors) ?? 'stainless_steel');
            $holes = (int) ($holes ?? settings::read('default_holes', $errors) ?? 0);
            $diameter = (int) ($diameter ?? settings::read('default_diameter', $errors) ?? 0);

            // Стоисмость киловатта электроэнергии
            $electricity = settings::read('electricity', $errors) ?? 6.5;

            // Потребляемая электроэнергия станком (квт/ч)
            $power = settings::read('laser_power', $errors) ?? 2;

            // 1 мм толщина = 220 мм/с рез у нержавеющей стали
            $speed = 220;

            // Вычисление площади
            $area = $width * $height;

            // Коэффициент сложности
            $coefficient = ($area <= (settings::read('coefficient_area_less', $errors) ?? 10000) || $area >= (settings::read('coefficient_area_more', $errors) ?? 100000) ? (settings::read('coefficient_area_degree', $errors) ?? 0.2) : 0) + match ($complexity) {
                'easy' => (settings::read('coefficient_complexity_easy', $errors) ?? 0.8),
                'medium' => (settings::read('coefficient_complexity_medium', $errors) ?? 1),
                'hard' => (settings::read('coefficient_complexity_hard', $errors) ?? 1.2),
                default => (settings::read('coefficient_complexity_medium', $errors) ?? 1)
            };

            // Расчет длины реза (мм)
            eval(settings::read('laser_formula_cutting', $errors) ?? '$cutting = 3.14 * $diameter * $holes + ($width * 2 + $height * 2) * $coefficient;');

            // Скорость реза в час (мм/с)
            eval(settings::read('laser_formula_speed', $errors) ?? '$speed = 3600 * $speed;');

            // Стоимость реза в час
            eval(settings::read('laser_formula_hour', $errors) ?? '$hour = $electricity * $power;');

            // Стоимость 1 миллиметра реза
            eval(settings::read('laser_formula_millimeter', $errors) ?? '$millimeter = $hour / $speed;');

            // Cтанок (стоимость работы)
            eval(settings::read('laser_formula_machine', $errors) ?? '$machine = $cutting * $millimeter * $amount;');

            var_dump($company, $complexity, $width, $height, $length, $amount, $metal, $holes, $diameter);

            if ($company) {
                // Юридическое лицо

                $min = settings::read('manager_entity_min', $errors) ?? 1;
                $max = settings::read('manager_entity_max', $errors) ?? 7;
                $average = ($min + $max) / 2;

                // Менеджер (стоимость работы)
                $manager = (settings::read('manager_entity_hour', $errors) ?? 200) * match ($complexity) {
                    'easy' => $min,
                    'medium' => $average,
                    'hard' => $max,
                    default => $average
                };

                $min = settings::read('engineer_entity_min', $errors) ?? 2;
                $max = settings::read('engineer_entity_max', $errors) ?? 72;
                $average = ($min + $max) / 2;

                // Инженер (стоимость работы)
                $engineer = (settings::read('engineer_entity_hour', $errors) ?? 400) * match ($complexity) {
                    'easy' => $min,
                    'medium' => $average,
                    'hard' => $max,
                    default => $average
                };

                $min = settings::read('operator_entity_min', $errors) ?? 0.33;
                $max = settings::read('operator_entity_max', $errors) ?? 16;
                $average = ($min + $max) / 2;

                // Оператор
                $operator = (settings::read('operator_entity_hour', $errors) ?? 200) * match ($complexity) {
                    'easy' => $min,
                    'medium' => $average,
                    'hard' => $max,
                    default => $average
                };
            } else {
                // Физическое лицо

                $min = settings::read('manager_individual_min', $errors) ?? 1;
                $max = settings::read('manager_individual_max', $errors) ?? 3;
                $average = ($min + $max) / 2;

                // Менеджер (стоимость работы)
                $manager = (settings::read('manager_individual_hour', $errors) ?? 200) * match ($complexity) {
                    'easy' => $min,
                    'medium' => $average,
                    'hard' => $max,
                    default => $average
                };

                $min = settings::read('manager_individual_min', $errors) ?? 2;
                $max = settings::read('manager_individual_max', $errors) ?? 10;
                $average = ($min + $max) / 2;

                // Инженер (стоимость работы)
                $engineer = (settings::read('engineer_individual_hour', $errors) ?? 300) * match ($complexity) {
                    'easy' => $min,
                    'medium' => $average,
                    'hard' => $max,
                    default => $average
                };

                $min = settings::read('manager_individual_min', $errors) ?? 0.33;
                $max = settings::read('manager_individual_max', $errors) ?? 7;
                $average = ($min + $max) / 2;

                // Оператор (стоимость работы)
                $operator = (settings::read('operator_individual_hour', $errors) ?? 200) * match ($complexity) {
                    'easy' => $min,
                    'medium' => $average,
                    'hard' => $max,
                    default => $average
                };
            }
        } catch (exception $e) {
            // Запись в журнал ошибок
            $errors[] = $e->getMessage();
        }

        return [[$machine], [$manager], [$engineer], [$operator]];
    }
}
